@@ -381,9 +381,17 @@ class S3FileSystem(AsyncFileSystem):
     @staticmethod
     def close_session(loop, s3):
         if loop is not None and loop.is_running():
-            sync(loop, s3.__aexit__, None, None, None, timeout=0.1)
-        else:
-            s3._endpoint.http_session._connector._close
+            try:
+                sync(loop, s3.__aexit__, None, None, None, timeout=0.1)
+                return
+            except TimeoutError:
+                pass
+        try:
+            # close the actual socket
+            s3._client._endpoint.http_session._connector._close()
+        except AttributeError:
+            # but during shutdown, it may have gone
+            pass
 
     async def _get_delegated_s3pars(self, exp=3600):
         """Get temporary credentials from STS, appropriate for sending across a
@@ -576,26 +584,38 @@ class S3FileSystem(AsyncFileSystem):
                 out = []
         dirs = []
         sdirs = set()
+        thisdircache = {}
         for o in out:
             par = self._parent(o["name"])
             if par not in self.dircache:
                 if par not in sdirs:
                     sdirs.add(par)
                     if len(path) <= len(par):
-                        dirs.append(
-                            {
-                                "Key": self.split_path(par)[1],
-                                "Size": 0,
-                                "name": par,
-                                "StorageClass": "DIRECTORY",
-                                "type": "directory",
-                                "size": 0,
-                            }
-                        )
-                    self.dircache[par] = []
+                        d = {
+                            "Key": self.split_path(par)[1],
+                            "Size": 0,
+                            "name": par,
+                            "StorageClass": "DIRECTORY",
+                            "type": "directory",
+                            "size": 0,
+                        }
+                        dirs.append(d)
+                    thisdircache[par] = []
+                    ppar = self._parent(par)
+                    if ppar in thisdircache:
+                        if d not in thisdircache[ppar]:
+                            thisdircache[ppar].append(d)
             if par in sdirs:
-                self.dircache[par].append(o)
-
+                thisdircache[par].append(o)
+        for k, v in thisdircache.items():
+            if k in self.dircache:
+                prev = self.dircache[k]
+                names = [p["name"] for p in prev]
+                for file in v:
+                    if v["name"] not in names:
+                        prev.append(v)
+            else:
+                self.dircache[k] = v
         if withdirs:
             out = sorted(out + dirs, key=lambda x: x["name"])
         if detail:
@@ -714,6 +734,7 @@ class S3FileSystem(AsyncFileSystem):
         if path in ["", "/"]:
             # the root always exists, even if anon
             return True
+        path = self._strip_protocol(path)
         bucket, key, version_id = self.split_path(path)
         if key:
             try:
@@ -888,6 +909,67 @@ class S3FileSystem(AsyncFileSystem):
         finally:
             body.close()
 
+    async def _simple_info(self, path):
+        bucket, key, _ = self.split_path(path)
+        prefix = key.strip("/")
+
+        out = await self._call_s3(
+            "list_objects_v2",
+            self.kwargs,
+            Bucket=bucket,
+            Prefix=prefix,
+            Delimiter="/",
+            MaxKeys=1,
+            **self.req_kw,
+        )
+        # This method either can return the info blob for the object if it
+        # exists, or return None if it doesn't exist as a file and it's
+        # existence as a directory is undeterminable with this call. If we
+        # can determine that no file or directory exists on the S3 with this
+        # prefix, then we can safely raise the FileNotFoundError.
+        [file] = out.get("Contents", [None])
+        if file and file["Key"] == prefix:
+            return {
+                "ETag": file["ETag"],
+                "Key": "/".join([bucket, key]),
+                "LastModified": file["LastModified"],
+                "Size": file["Size"],
+                "size": file["Size"],
+                "name": "/".join([bucket, key]),
+                "type": "file",
+                "StorageClass": "STANDARD",
+                "VersionId": file.get("VersionId"),
+            }
+        elif not (file or out.get("KeyCount", 0) or out.get("CommonPrefixes", [])):
+            raise FileNotFoundError(path)
+
+    async def _version_aware_info(self, path, version_id):
+        bucket, key, _ = self.split_path(path)
+
+        try:
+            out = await self._call_s3(
+                "head_object",
+                self.kwargs,
+                Bucket=bucket,
+                Key=key,
+                **version_id_kw(version_id),
+                **self.req_kw,
+            )
+        except FileNotFoundError:
+            return None
+
+        return {
+            "ETag": out["ETag"],
+            "Key": "/".join([bucket, key]),
+            "LastModified": out["LastModified"],
+            "Size": out["ContentLength"],
+            "size": out["ContentLength"],
+            "name": "/".join([bucket, key]),
+            "type": "file",
+            "StorageClass": "STANDARD",
+            "VersionId": out.get("VersionId"),
+        }
+
     async def _info(self, path, bucket=None, key=None, refresh=False, version_id=None):
         path = self._strip_protocol(path)
         bucket, key, path_version_id = self.split_path(path)
@@ -908,27 +990,23 @@ class S3FileSystem(AsyncFileSystem):
             return {"name": path, "size": 0, "type": "directory"}
         if key:
             try:
-                out = await self._call_s3(
-                    "head_object",
-                    self.kwargs,
-                    Bucket=bucket,
-                    Key=key,
-                    **version_id_kw(version_id),
-                    **self.req_kw,
-                )
-                return {
-                    "ETag": out["ETag"],
-                    "Key": "/".join([bucket, key]),
-                    "LastModified": out["LastModified"],
-                    "Size": out["ContentLength"],
-                    "size": out["ContentLength"],
-                    "name": "/".join([bucket, key]),
-                    "type": "file",
-                    "StorageClass": "STANDARD",
-                    "VersionId": out.get("VersionId"),
-                }
-            except FileNotFoundError:
-                pass
+                # The HeadObject calls are really expensive on objects
+                # compared to a ListObjectsV2 call with a max_keys=1. The
+                # disadvantage of the list objects is that, it doesn't yield
+                # a list of versions which might be needed on version_aware filesystems.
+                # So we switch between a normal HeadObject call (when version_aware is set)
+                # and ListObjectsV2 (when it is not).
+                if self.version_aware:
+                    out = await self._version_aware_info(path, version_id)
+                else:
+                    try:
+                        out = await self._simple_info(path)
+                    except PermissionError:
+                        # If the permissions aren't enough for scanning a prefix
+                        # then fall back to using normal HEAD_OBJECT
+                        out = await self._version_aware_info(path, version_id)
+                if out:
+                    return out
             except ClientError as e:
                 raise translate_boto_error(e, set_cause=False)
 
@@ -1440,7 +1518,7 @@ class S3FileSystem(AsyncFileSystem):
         info = await self._info(path1, bucket, key, version_id=vers)
         size = info["size"]
 
-        _, _, parts_suffix = info["ETag"].strip('"').partition("-")
+        _, _, parts_suffix = info.get("ETag", "").strip('"').partition("-")
         if preserve_etag and parts_suffix:
             await self._copy_etag_preserved(
                 path1, path2, size, total_parts=int(parts_suffix)
