@@ -70,6 +70,10 @@ def s3_base():
     import shlex
     import subprocess
 
+    if "AWS_SECRET_ACCESS_KEY" not in os.environ:
+        os.environ["AWS_SECRET_ACCESS_KEY"] = "foo"
+    if "AWS_ACCESS_KEY_ID" not in os.environ:
+        os.environ["AWS_ACCESS_KEY_ID"] = "foo"
     proc = subprocess.Popen(shlex.split("moto_server s3 -p %s" % port))
 
     timeout = 5
@@ -871,10 +875,10 @@ def test_errors_cause_preservings(monkeypatch, s3):
 
     assert type(exc.value.__cause__).__name__ == "NoSuchBucket"
 
-    async def head_object(*args, **kwargs):
+    async def list_objects_v2(*args, **kwargs):
         raise NoCredentialsError
 
-    monkeypatch.setattr(type(s3.s3), "head_object", head_object)
+    monkeypatch.setattr(type(s3.s3), "list_objects_v2", list_objects_v2)
 
     # Since the error is not translate, the __cause__ would
     # be None
@@ -2020,3 +2024,78 @@ def test_token_paths(s3):
         storage_options={"client_kwargs": {"endpoint_url": endpoint_uri}},
     )
     assert files
+
+
+def test_same_name_but_no_exact(s3):
+    s3.touch(test_bucket_name + "/very/similiar/prefix1")
+    s3.touch(test_bucket_name + "/very/similiar/prefix2")
+    s3.touch(test_bucket_name + "/very/similiar/prefix3/something")
+    assert not s3.exists(test_bucket_name + "/very/similiar/prefix")
+    assert not s3.exists(test_bucket_name + "/very/similiar/prefi")
+    assert not s3.exists(test_bucket_name + "/very/similiar/pref")
+
+    assert s3.exists(test_bucket_name + "/very/similiar/")
+    assert s3.exists(test_bucket_name + "/very/similiar/prefix1")
+    assert s3.exists(test_bucket_name + "/very/similiar/prefix2")
+    assert s3.exists(test_bucket_name + "/very/similiar/prefix3")
+    assert s3.exists(test_bucket_name + "/very/similiar/prefix3/")
+    assert s3.exists(test_bucket_name + "/very/similiar/prefix3/something")
+
+    assert not s3.exists(test_bucket_name + "/very/similiar/prefix3/some")
+
+    s3.touch(test_bucket_name + "/starting/very/similiar/prefix")
+
+    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix1")
+    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix2")
+    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix3")
+    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix3/")
+    assert not s3.exists(test_bucket_name + "/starting/very/similiar/prefix3/something")
+
+    assert s3.exists(test_bucket_name + "/starting/very/similiar/prefix")
+    assert s3.exists(test_bucket_name + "/starting/very/similiar/prefix/")
+
+
+def test_info_with_permission_error_for_list_objects(monkeypatch, s3):
+    s3.touch(test_bucket_name + "/very/similiar/prefix")
+
+    async def list_objects_v2(*args, **kwargs):
+        if kwargs.pop("Prefix").endswith("/"):
+            return {}
+        else:
+            raise PermissionError
+
+    monkeypatch.setattr(type(s3.s3), "list_objects_v2", list_objects_v2)
+    assert not s3.exists(test_bucket_name + "/very/similiar/prefix1")
+    assert s3.exists(test_bucket_name + "/very/similiar/prefix")
+    assert s3.info(test_bucket_name + "/very/similiar/prefix")["type"] == "file"
+
+
+def test_leading_forward_slash(s3):
+    s3.touch(test_bucket_name + "/some/file")
+    assert s3.ls(test_bucket_name + "/some/")
+    assert s3.exists(test_bucket_name + "/some/file")
+    assert s3.exists("s3://" + test_bucket_name + "/some/file")
+
+
+def test_lsdir(s3):
+    # https://github.com/dask/s3fs/issues/475
+    s3.find(test_bucket_name)
+
+    d = test_bucket_name + "/test"
+    assert d in s3.ls(test_bucket_name)
+
+
+def test_copy_file_without_etag(s3, monkeypatch):
+
+    s3.touch(test_bucket_name + "/copy_tests/file")
+    s3.ls(test_bucket_name + "/copy_tests/")
+
+    [file] = s3.dircache[test_bucket_name + "/copy_tests"]
+
+    assert file["name"] == test_bucket_name + "/copy_tests/file"
+    file.pop("ETag")
+
+    assert s3.info(file["name"]).get("ETag", None) is None
+
+    s3.cp_file(file["name"], test_bucket_name + "/copy_tests/file2")
+    assert s3.info(test_bucket_name + "/copy_tests/file2")["ETag"] is not None
